package com.learning.block;

import com.learning.Learning;

import java.util.function.BiFunction;
import java.util.function.Function;

import net.minecraft.block.*;
import net.minecraft.registry.Registries;
import net.minecraft.registry.Registry;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.util.Identifier;

public class CustomBlocks {
    public static void registerBlocks() {
        Learning.LOGGER.info("Registering blocks...");
    }

    public static final Block TEST_BLOCK = register("test_block");
    public static final Block DAMAGE_BLOCK = register("damage_block", DamageBlock::new, AbstractBlock.Settings.create().requiresTool());
    public static final Block TRANSFORM_BLOCK = register("transform_block", TransformBlock::new);
    public static final Block DENSITY_GLASS = register("density_glass");

    public static final Block DENSITY_GLASS_STAIRS = register("density_glass_stairs", StairsBlock::new,
            DENSITY_GLASS.getDefaultState(), AbstractBlock.Settings.create().requiresTool().strength(2f));
    public static final Block DENSITY_GLASS_FENCE = register("density_glass_fence", FenceBlock::new, AbstractBlock.Settings.create().requiresTool().strength(2f));
    public static final Block DENSITY_GLASS_FENCE_GATE = register("density_glass_gate", FenceGateBlock::new,
            WoodType.OAK, AbstractBlock.Settings.create().requiresTool().strength(2f));
    public static final Block DENSITY_GLASS_SLAB = register("density_glass_slab", SlabBlock::new, AbstractBlock.Settings.create().requiresTool().strength(2f).noCollision());
    public static final Block DENSITY_GLASS_DOOR = register("density_glass_door", (AbstractBlock.Settings blockSettings) -> new DoorBlock(BlockSetType.IRON, blockSettings),
            AbstractBlock.Settings.create().requiresTool().strength(2f).nonOpaque());

    public static Block register(String blockId) {
        return register(blockId, Block::new, AbstractBlock.Settings.create().strength(2f));
    }

    public static Block register(String blockId, Function<AbstractBlock.Settings, Block> factory) {
        return register(blockId, factory, AbstractBlock.Settings.create());
    }

    // for BlockState, BlockSettings register
    public static Block register(String blockId, BiFunction<BlockState, AbstractBlock.Settings, Block> factory, BlockState blockState, AbstractBlock.Settings settings) {
        RegistryKey<Block> key = keyOf(blockId);
        return register(key, factory.apply(blockState, settings.registryKey(key)));
    }

    // for WoodType, BlockSettings register
    public static Block register(String blockId, BiFunction<WoodType, AbstractBlock.Settings, Block> factory, WoodType woodType, AbstractBlock.Settings settings) {
        RegistryKey<Block> key = keyOf(blockId);
        return register(key, factory.apply(woodType, settings.registryKey(key)));
    }

    public static Block register(String blockId, Function<AbstractBlock.Settings, Block> factory, AbstractBlock.Settings settings) {
        RegistryKey<Block> key = keyOf(blockId);
        return register(key, factory.apply(settings.registryKey(key)));
    }

    public static Block register(RegistryKey<Block> key, Block block) {
        return Registry.register(Registries.BLOCK, key, block);
    }

    private static RegistryKey<Block> keyOf(String blockId) {
        return RegistryKey.of(RegistryKeys.BLOCK, Identifier.of(Learning.MOD_ID, blockId));
    }
}
