package com.learning.item;


import java.util.List;
import java.util.function.Function;

import com.learning.Learning;
import com.learning.block.CustomBlocks;
import com.learning.component.Foods;

import net.minecraft.item.BlockItem;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.Items;
import net.minecraft.item.tooltip.TooltipType;
import net.minecraft.registry.Registries;
import net.minecraft.registry.Registry;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.text.Text;
import net.minecraft.util.Formatting;
import net.minecraft.util.Identifier;
import net.minecraft.block.Block;
import net.minecraft.component.type.ConsumableComponents;

public class CustomItems {
    public static void registerItems() {
        Learning.LOGGER.info("Registering Items...");
    }

    public static Item TEST_ITEM = register("test_item");

    public static Item TOAST = register("toast", Toast::new, new Item.Settings().food(Foods.toastFood));
    public static Item BAKED_TOAST = register("baked_toast", BakedToast::new, new Item.Settings().food(Foods.bakedToastFood, ConsumableComponents.food().consumeSeconds(0.8f).build()));
    public static Item OVERCOOKED_TOAST = register(keyOf("overcooked_toast"), new Item(new Item.Settings().registryKey(keyOf("overcooked_toast"))) {
        @Override
        public void appendTooltip(ItemStack stack, TooltipContext context, List<Text> tooltip, TooltipType type) {
            tooltip.add(Text.translatable("toolTip.learning.overcooked_toast").formatted(Formatting.GRAY));
            super.appendTooltip(stack, context, tooltip, type);
        }
    });

    public static Item DAMAGE_BLOCK = register(CustomBlocks.DAMAGE_BLOCK, false);
    public static Item TRANSFORM_BLOCK = register(CustomBlocks.TRANSFORM_BLOCK, false);
    public static Item DENSITY_GLASS = register(CustomBlocks.DENSITY_GLASS, false);
    public static Item DENSITY_GLASS_DOOR = register(CustomBlocks.DENSITY_GLASS_DOOR, false);
    public static Item DENSITY_GLASS_STAIRS = register(CustomBlocks.DENSITY_GLASS_STAIRS, false);
    public static Item DENSITY_GLASS_FENCE = register(CustomBlocks.DENSITY_GLASS_FENCE, false);
    public static Item DENSITY_GLASS_SLAB = register(CustomBlocks.DENSITY_GLASS_SLAB, false);
    public static Item DENSITY_GLASS_FENCE_GATE = register(CustomBlocks.DENSITY_GLASS_FENCE_GATE, false);



    private static Item register(Block block, Boolean useItemTranslateKey) {  // Block register
        if (Registries.BLOCK.getKey(block).isPresent()) {
            RegistryKey<Item> key = keyOf(Registries.BLOCK.getKey(block).get());
            Item.Settings settings = new Item.Settings().registryKey(key);
            if (!useItemTranslateKey) {
                settings.useBlockPrefixedTranslationKey();
            }
            return register(key, new BlockItem(block, settings));
        }
        return Items.AIR;
    }

    private static Item register(String itemId) {
        return register(itemId, new Item.Settings());
    }

    private static Item register(String itemId, Item.Settings settings) {
        return register(itemId, Item::new, settings);
    }
  
    private static Item register(String itemId, Function<Item.Settings, Item> factory, Item.Settings settings) {
        RegistryKey<Item> key = keyOf(itemId);    
        return register(key, factory, settings);
    }

    private static Item register(RegistryKey<Item> key, Function<Item.Settings, Item> factory, Item.Settings settings) {
        return register(key, factory.apply(settings.registryKey(key)));
    }

    private static Item register(RegistryKey<Item> key, Item item) {
        return Registry.register(Registries.ITEM, key, item);
    }

    private static RegistryKey<Item> keyOf(String itemId) {
        return RegistryKey.of(RegistryKeys.ITEM, Identifier.of(Learning.MOD_ID, itemId));
    }

    private static RegistryKey<Item> keyOf(RegistryKey<Block> blockKey) {
        return RegistryKey.of(RegistryKeys.ITEM, blockKey.getValue());
    }
}
